﻿using Falcon.SugarApi.BackTask;
using Microsoft.AspNetCore.Mvc.ApplicationParts;
using Microsoft.Extensions.Configuration;
using Microsoft.Extensions.DependencyInjection;
using System;
using System.Linq;
using System.Reflection;

namespace Falcon.SugarApi.Plugin.Client
{
    /// <summary>
    /// 服务扩展 客户端调用
    /// </summary>
    public static class IServiceCollectionExtend
    {
        /// <summary>
        /// 注册当前程序集中的控制器
        /// </summary>
        /// <param name="services">服务集合</param>
        /// <param name="assembly">要注册的程序集</param>
        /// <returns>服务集合</returns>
        public static IServiceCollection AddPluginsController(this IServiceCollection services,Assembly assembly) {
            var part = new AssemblyPart(assembly);
            services.AddControllers().ConfigureApplicationPartManager(apm => {
                if(apm.ApplicationParts.Any(a => a.Name == part.Name)) {
                    return;
                }
                apm.ApplicationParts.Add(part);
                
            });
            return services;
        }

        /// <summary>
        /// 注册后台背景任务
        /// </summary>
        /// <typeparam name="T">任务类型</typeparam>
        /// <param name="services">服务集合</param>
        /// <returns>服务集合</returns>
        public static IServiceCollection AddPluginsBackTask<T>(this IServiceCollection services)
            where T : BackgroundLongTask => services.AddHostedService<T>();

        /// <summary>
        /// 注册插件配置
        /// </summary>
        /// <typeparam name="T">配置类型</typeparam>
        /// <param name="services">服务集合</param>
        /// <param name="options">配置实体</param>
        /// <returns>服务集合</returns>
        public static IServiceCollection AddPluginOption<T>(this IServiceCollection services,T options)
            where T : class => services.AddSingleton(options);

        /// <summary>
        /// 注册插件配置
        /// </summary>
        /// <typeparam name="T">配置类型</typeparam>
        /// <param name="services">服务集合</param>
        /// <param name="config">可选，应用配置节点</param>
        /// <param name="optionBuilder">可选，配置创建器</param>
        /// <returns>服务集合</returns>
        public static IServiceCollection AddPluginOption<T>(this IServiceCollection services,IConfigurationSection? config,Action<T>? optionBuilder)
            where T : class, new() {
            var options = config == null ? new T() : config.Get<T>();
            options = options ?? throw new Exception("Get Options fail!");
            optionBuilder?.Invoke(options);
            return services.AddPluginOption(options);
        }
    }
}
